function [info, present] = dataset_sleipnerplumes()
% Info function for CO2lab dataset. Use getDatasetInfo or getAvailableDatasets for practical purposes.

%{
Copyright 2009-2017 SINTEF ICT, Applied Mathematics.

This file is part of The MATLAB Reservoir Simulation Toolbox (MRST).

MRST is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MRST is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MRST.  If not, see <http://www.gnu.org/licenses/>.
%}
    [info, present] = datasetInfoStruct(...
        'name', 'SleipnerPlumes', ...
        'website', 'http://www.ieaghg.org/networks/modelling-network', ...
        'fileurl', 'https://www.sintef.no/contentassets/124f261f170947a6bc51dd76aea66129/sleipner_plume_thicknesses.zip', ...
        'description', ['This dataset does not contain the IEAGHG Sleipner', ...
        ' Benchmark model. Rather, this dataset includes the CO2 plume', ...
        ' thicknesses of the 2001, 2004, 2006, and 2010 observation years,', ...
        ' which were taken from literature: 2001, 2004, 2006 comes from', ...
        ' Figure 6 in Chadwick & Noy, "History-matching flow simulations', ...
        ' and time-lapse seismic data from the Sleipner CO2 plume"', ...
        ' (submitted to 7th Petroleum Geology Conference Proceedings),', ...
        ' DOI: 10.1144/0071171, and', ...
        ' 2010 comes from Figure 14 in Furre & Eiken, 2014, "Dual sensor', ...
        ' streamer technology used in Sleipner CO2 injection monitoring",', ...
        ' Geophysical Prospecting, DOI: 10.1111/1365-2478.12120', ...
        ' To access the model, go to the IEAGHG modeling network webpage', ...
        ' (click "webpage" below) and follow the instructions there. (Note', ...
        ' must register as a member of the modeling network first).'], ...
        'filesize',    1, ...
        'examples', { 'co2lab:calibrateSleipner', ...
                      }, ...                 
        'modelType', '***Plume data only. No' ...
         );
end